package cerebral.colors;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.plaf.metal.MetalButtonUI;

import prefuse.util.ColorLib;

public class LinearRangeEditor extends JPanel implements
		PropertyChangeListener, FocusListener {

	private static final long serialVersionUID = 1L;
	private LinearRange lr;
	private JFormattedTextField minVal, maxVal;
	private JButton colorButton, colorMaxButton;

	public LinearRangeEditor() {
		setBackground(Color.white);
		setLayout(new FlowLayout(FlowLayout.LEFT));
		colorButton = new JButton("");
		colorButton.setUI(new MetalButtonUI());
		colorButton.setOpaque(true);
		colorButton.setPreferredSize(new Dimension(15, 15));

		colorButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (lr != null) {
					Color selectedColor = JColorChooser.showDialog(null,
							"Choose color for the lowest value in the range",
							ColorLib.getColor(lr.colorMin));
					if (selectedColor != null) {
						lr.colorMin = ColorLib.color(selectedColor);
						colorButton.setBackground(ColorLib
								.getColor(lr.colorMin));
						firePropertyChange("color", null, null);
					}
				}
			}
		});
		add(colorButton);
		JLabel minLabel = new JLabel("Minimum");
		minLabel.setLabelFor(minVal);
		add(minLabel);

		minVal = new JFormattedTextField("#.####");
		minVal.setEditable(true);
		minVal.setValue(new Double(0.0));
		minVal.addPropertyChangeListener("value", this);
		minVal.addFocusListener(this);
		add(minVal);

		maxVal = new JFormattedTextField("#.####");
		maxVal.setEditable(true);
		maxVal.setValue(new Double(0.0));
		maxVal.addPropertyChangeListener("value", this);
		maxVal.addFocusListener(this);
		add(maxVal);
		JLabel maxLabel = new JLabel("Maximum");
		maxLabel.setLabelFor(maxVal);
		add(maxLabel);

		colorMaxButton = new JButton("");
		colorMaxButton.setUI(new MetalButtonUI());
		colorMaxButton.setOpaque(true);
		colorMaxButton.setPreferredSize(new Dimension(15, 15));
		colorMaxButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (lr != null) {
					Color selectedColor = JColorChooser.showDialog(null,
							"Choose color for the highest value in the range",
							ColorLib.getColor(lr.colorMax));
					if (selectedColor != null) {
						lr.colorMax = ColorLib.color(selectedColor);
						colorMaxButton.setBackground(ColorLib
								.getColor(lr.colorMax));
						firePropertyChange("color", null, null);
					}
				}
			}
		});
		add(colorMaxButton);
	}

	public void setFocus() {
		minVal.requestFocus();
	}

	public void commit() {
		commitMin();
		commitMax();
	}

	private void commitMin() {
		try {
			if (lr != null) {
				double newMin = Double.parseDouble(minVal.getText());
				if (newMin < lr.next().getMin()) {
					if (lr.prev() != null) {
						if (newMin < lr.prev().getMin()) {
							// expanded past previous. Disallow
							minVal.setForeground(Color.red);
							return;
						}
					}
					lr.setMin(newMin);
					minVal.setForeground(Color.black);
					firePropertyChange("min", null, null);
				} else {
					minVal.setForeground(Color.red);
				}
			}
		} catch (java.lang.NumberFormatException x) {
		}
	}

	private void commitMax() {
		try {

			if (lr != null) {
				double newMax = Double.parseDouble(maxVal.getText());
				if (newMax > lr.getMin()) {
					if (lr.next().next() != null) {
						if (newMax > lr.next().next().getMin()) {
							maxVal.setForeground(Color.red);
							return;
						}
					}
					lr.next().setMin(newMax);
					maxVal.setForeground(Color.black);
					firePropertyChange("max", null, null);
				} else {
					maxVal.setForeground(Color.red);
				}
			}
		} catch (java.lang.NumberFormatException x) {
		}

	}

	public void setLinearRange(LinearRange lrIn) {
		
		if (lr != null){
			lr.setBorder(BorderFactory.createEmptyBorder());
		}
		lr = lrIn;
		if (lr != null){
			lr.setBorder(BorderFactory.createEtchedBorder());
		
			lr.setBackground(Color.yellow);
			colorButton.setBackground(ColorLib.getColor(lr.colorMin));
			colorMaxButton.setBackground(ColorLib.getColor(lr.colorMax));

			minVal.setValue(new Double(lr.getMin()));
			maxVal.setValue(new Double(lr.next().getMin()));
		}

	}

	public void propertyChange(PropertyChangeEvent e) {
		if (e.getPropertyName() == "value") {
			if (e.getSource() == minVal) {
				commitMin();
			} else if (e.getSource() == maxVal) {
				commitMax();
			}
		}
	}

	public void focusGained(FocusEvent e) {
		if (e.getSource() instanceof JFormattedTextField) {
			JFormattedTextField text = (JFormattedTextField) e.getSource();
			String oldText = text.getText();
			text.setText("");
			text.setText(oldText);
			text.selectAll();
		}
	}

	public void focusLost(FocusEvent e) {
	}
}