package cerebral.controls;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import prefuse.Visualization;
import prefuse.controls.ControlAdapter;
import prefuse.data.Tuple;
import prefuse.data.tuple.TupleSet;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;
import publicdomain.BareBonesBrowserLaunch;
import cerebral.Cerebral;

public class PopupMenuControl extends ControlAdapter implements ActionListener {
  
    private static final String ADD_CURRENT = "addCurrent";

    private static final String ADD_NEW = "addNew";

    private static final String REMOVE_CURRENT = "removeCurrent";

    private static final String PIN_NODES = "pinNodes";
    
    private static final String UNPIN_NODES = "unpinNodes";
    

    private static final String INNATE_DB = "innateDB";

    private VisualItem activeItem;

    private TupleSet activeSelection;

    private Cerebral cytoPrefuse;

    private Visualization vis;

    public PopupMenuControl(Cerebral cytoPrefuse, Visualization vis) {
        this.cytoPrefuse = cytoPrefuse;
        this.vis = vis;
    }

    public void itemClicked(VisualItem item, MouseEvent e) {
        if (SwingUtilities.isRightMouseButton(e)) {
            JPopupMenu menu = new JPopupMenu();

            if (item instanceof NodeItem || item instanceof EdgeItem) {
                activeItem = item;
                activeSelection = vis.getFocusGroup(Cerebral.SELECTED);
                String URL = activeItem.getString(Cerebral.INNATEDBREF);
                if (URL != null) {
                    JMenuItem linkToInnateDB;
                    if (item instanceof NodeItem) {
                        linkToInnateDB = new JMenuItem("Lookup "
                                + activeItem.getString(Cerebral.LABEL)
                                + " in InnateDB");
                    } else {
                        linkToInnateDB = new JMenuItem(
                                "Lookup interaction in InnateDB");
                    }

                    linkToInnateDB.setActionCommand(INNATE_DB);
                    linkToInnateDB.addActionListener(this);
                    menu.add(linkToInnateDB);
                    menu.addSeparator();
                }

                
                String messagePrefix = "Add ";
                if (activeSelection.getTupleCount() > 1) {
                    messagePrefix += "selected items ";
                }
                messagePrefix += "to ";
                if (cytoPrefuse.groupsManager.getNumGroups() > 0) {
                    JMenuItem addToGroup = new JMenuItem(messagePrefix
                            + cytoPrefuse.groupsManager.getCurrentGroupName());
                    addToGroup.setActionCommand(ADD_CURRENT);
                    addToGroup.addActionListener(this);
                    menu.add(addToGroup);

                    String removePrefix = "Remove ";
                    if (activeSelection.getTupleCount() > 1) {
                        removePrefix += "selected items ";
                    }
                    removePrefix += "from ";

                    JMenuItem removeFromGroup = new JMenuItem(removePrefix
                            + cytoPrefuse.groupsManager.getCurrentGroupName());
                    removeFromGroup.setActionCommand(REMOVE_CURRENT);
                    removeFromGroup.addActionListener(this);
                    menu.add(removeFromGroup);

                    menu.addSeparator();

                }

                for (int i = 0; i < cytoPrefuse.groupsManager.getNumGroups(); i++) {
                    String groupName = cytoPrefuse.groupsManager
                            .getGroupName(i);
                    if (!groupName.equals(cytoPrefuse.groupsManager
                            .getCurrentGroupName())) {
                        JMenuItem addOtherGroup = new JMenuItem(messagePrefix
                                + groupName);
                        addOtherGroup.setActionCommand(groupName);
                        addOtherGroup.addActionListener(this);
                        menu.add(addOtherGroup);
                    }
                }
                if (cytoPrefuse.groupsManager.getNumGroups() > 1) {
                    menu.addSeparator();
                }

                JMenuItem addToNewGroup = new JMenuItem(messagePrefix
                        + "a new group");

                addToNewGroup.setActionCommand(ADD_NEW);
                addToNewGroup.addActionListener(this);
                menu.add(addToNewGroup);

                if (item instanceof NodeItem) {
                    menu.addSeparator();
                    String plural = "";
                    if (activeSelection.getTupleCount() <= 1
                            || allNodesAreSelected(activeSelection)) {
                        plural += "node";
                    } else {
                        plural += "selected nodes";
                    }

                    boolean pinned = item.getBoolean(Cerebral.PINNED);
                    if (pinned){
                        JMenuItem unpinNodes = new JMenuItem("Unpin "+plural);
                        unpinNodes.setActionCommand(UNPIN_NODES);
                        unpinNodes.addActionListener(this);
                        menu.add(unpinNodes);
                    }else{                        
                        JMenuItem pinNodes = new JMenuItem("Pin " + plural);
                        pinNodes.setActionCommand(PIN_NODES);
                        pinNodes.addActionListener(this);
                        menu.add(pinNodes);
                    }
                }

            }
            menu.show(e.getComponent(), e.getX(), e.getY());

        }
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals(INNATE_DB)) {
            String URL = activeItem.getString(Cerebral.INNATEDBREF);
            BareBonesBrowserLaunch.openURL(URL);
        } else if (e.getActionCommand().equals(ADD_CURRENT)) {
            addItemsToCurrentGroup();
        } else if (e.getActionCommand().equals(ADD_NEW)) {
            String newName = cytoPrefuse.groupsManager.addGroup();
            addItemsToGroup(newName);
        } else if (e.getActionCommand().equals(REMOVE_CURRENT)) {
            removeItemsFromCurrentGroup();
        } else if (e.getActionCommand().equals(PIN_NODES)) {
            pinNodes(true);
        }else if (e.getActionCommand().equals(UNPIN_NODES)) {
            pinNodes(false);
        } else {
            addItemsToGroup(e.getActionCommand());
        }
    }

    private boolean allNodesAreSelected(TupleSet selectedItems) {
        Iterator nodesIt = vis.items(Cerebral.NODES);
        while (nodesIt.hasNext()) {
            if (!selectedItems.containsTuple((Tuple) nodesIt.next())) {
                return false;
            }
        }

        return true;
    }

    private void pinNodes(boolean state) {
        if (activeSelection.getTupleCount() > 0  && !allNodesAreSelected(activeSelection)) {
            Iterator it = activeSelection.tuples();
            while (it.hasNext()) {
                VisualItem item = (VisualItem) it.next();
                if (item instanceof NodeItem) {
                    item.setBoolean(Cerebral.PINNED, state);
                }
            }
        } else {
            if (activeItem instanceof NodeItem) {
                activeItem.setBoolean(Cerebral.PINNED, state);
            }
        }

    }

    private void addItemsToGroup(String group) {
        if (activeSelection.getTupleCount() > 0) {
            Set addSet = new HashSet();
            Iterator it = activeSelection.tuples();
            while (it.hasNext()) {
                addSet.add(it.next());
            }
            cytoPrefuse.groupsManager.addItemsToGroup(addSet, group);
        } else {
            cytoPrefuse.groupsManager.addItemToGroup(activeItem, group);
        }

    }

    private void removeItemsFromCurrentGroup() {
        if (activeSelection.getTupleCount() > 0) {
            Set removeSet = new HashSet();
            Iterator it = activeSelection.tuples();
            while (it.hasNext()) {
                removeSet.add(it.next());
            }
            cytoPrefuse.groupsManager.removeItemsFromGroup(removeSet);
        } else {
            cytoPrefuse.groupsManager.removeItemFromGroup(activeItem);
        }
    }

    private void addItemsToCurrentGroup() {
        if (activeSelection.getTupleCount() > 0) {
            Set addSet = new HashSet();
            Iterator it = activeSelection.tuples();
            while (it.hasNext()) {
                addSet.add(it.next());
            }
            cytoPrefuse.groupsManager.addItemsToGroup(addSet);
        } else {
            cytoPrefuse.groupsManager.addItemToGroup(activeItem);
        }
    }

}
