package cerebral.parallelCoordinates;

import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import prefuse.render.Renderer;
import prefuse.util.ColorLib;
import prefuse.visual.VisualItem;

public class IntervalRenderer implements Renderer {
    private Rectangle2D rect = new Rectangle2D.Float();

    private Rectangle2D clip = new Rectangle2D.Double();

    private YDisplay yDisplay;

    public IntervalRenderer(YDisplay yDisplay) {
        clip.setRect(-1000000, 10, 2000000, 100);
        this.yDisplay = yDisplay;
    }

    public boolean locatePoint(Point2D p, VisualItem item) {
        return item.getBounds().contains(p);
    }

    public void render(Graphics2D g, VisualItem item) {
        double x = item.getX();
        double top = item.getDouble(ParallelCoordinates.TOP);
        double bottom = item.getDouble(ParallelCoordinates.BOTTOM);
        double y = yDisplay.getPosition(top);
        double h = yDisplay.getPosition(bottom) - y;
        rect.setRect(x - 8, y, 13, h);
        g.setClip(clip);
        g.setColor(ColorLib.getColor(125, 28, 246, 100));
        g.fill(rect);
        g.setClip(null);
    }

    public void setBounds(VisualItem item) {
        double x = item.getX();
        double top = item.getDouble(ParallelCoordinates.TOP);
        double bottom = item.getDouble(ParallelCoordinates.BOTTOM);

        double y = yDisplay.getPosition(top);
        double h = yDisplay.getPosition(bottom) - y;

        item.setBounds(x - 8, y, x, h);
    }

    public void setBounds(int top, double height) {
        clip.setRect(-1000000, top, 20000000, height);
    }

}
