package cerebral.render;

import giny.view.EdgeView;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import prefuse.render.AbstractShapeRenderer;
import prefuse.visual.EdgeItem;
import prefuse.visual.VisualItem;
import cerebral.Cerebral;
import cerebral.EdgeBundler;

public class BundleEdgeRenderer extends AbstractShapeRenderer {
    protected Line2D line = new Line2D.Float();

    protected Rectangle2D cursorRect = new Rectangle2D.Float();

    protected AffineTransform arrowTrans = new AffineTransform();

    private Shape curArrow;

    protected Polygon deltaHead = getDeltaHead(8, 12);

    private Polygon diamondHead = getDiamond(8, 12);

    private Polygon arrowHead = getArrowHead(8, 12);

    private Polygon tHead = getT(12);

    private Shape circle = new Ellipse2D.Double(-4, 0, 8, 8);

    private static float[] segmentPoints = new float[6];

    private static AffineTransform identity = new AffineTransform();

    public BundleEdgeRenderer() {
    }

    public boolean locatePoint(Point2D p, VisualItem item) {
        cursorRect.setFrame(p.getX(), p.getY(), 4, 4);
        return EdgeBundler.edgeIntersectsRect(item, cursorRect);
    }

    public void setBounds(VisualItem item) {
        Shape shape = getShape(item);
        Rectangle2D r = shape.getBounds();
        if (r.getWidth() >0 && r.getHeight() > 0){
            item.setBounds(r.getMinX(), r.getMinY(), r.getWidth(), r.getHeight());
        }else{
            item.setBounds(r.getMinX(), r.getMinY(), 1,1);
            
        }
       // if (shape != null)
         //   GraphicsLib.setBounds(item, shape, getStroke(item));
    }

    protected Shape getRawShape(VisualItem item) {
        return (Shape) item.get(Cerebral.EDGE_SPLINE);
    }

    public void render(Graphics2D g, VisualItem item) {
        EdgeItem edge = (EdgeItem) item;
        if (edge.getSourceItem().isVisible()
                && edge.getTargetItem().isVisible()) {
            super.render(g, item);
            renderEnds(g, item);
        }
    }

    public void renderEnds(Graphics2D g, VisualItem item) {
        EdgeItem edge = (EdgeItem) item;
        if (edge.getSourceItem().isVisible()
                && edge.getTargetItem().isVisible()) {
            GeneralPath path = (GeneralPath) edge.get(Cerebral.EDGE_SPLINE);
            PathIterator pathIt = path.getPathIterator(identity);
            pathIt.currentSegment(segmentPoints);
            Point2D start = new Point2D.Float(segmentPoints[0],
                    segmentPoints[1]);
            pathIt.next();
            pathIt.currentSegment(segmentPoints);
            Point2D startAngle = new Point2D.Float(segmentPoints[0],
                    segmentPoints[1]);
            pathIt.next();

            drawArrow(g, edge, start, startAngle, item
                    .getInt(Cerebral.SOURCE_STYLE));

            Point2D endAngle = (Point2D) startAngle.clone();
            Point2D end = (Point2D) startAngle.clone();

            while (!pathIt.isDone()) {
                endAngle.setLocation(end);
                pathIt.currentSegment(segmentPoints);
                end.setLocation(segmentPoints[0], segmentPoints[1]);
                pathIt.next();

            }

            drawArrow(g, edge, end, endAngle, item
                    .getInt(Cerebral.TARGET_STYLE));

        }
    }

    private void drawArrow(Graphics2D g, EdgeItem edge, Point2D point,
            Point2D otherEnd, int style) {

        switch (style) {
        case (EdgeView.WHITE_CIRCLE):
            g.setColor(Color.white);
            drawArrow(g, circle, point, otherEnd);
            break;
        case (EdgeView.BLACK_CIRCLE):
            g.setColor(Color.black);
            drawArrow(g, circle, point, otherEnd);
            break;
        case (EdgeView.EDGE_COLOR_CIRCLE):
            g.setColor((Color) edge.get(Cerebral.FILL_COLOR));
            drawArrow(g, circle, point, otherEnd);
            break;

        case (EdgeView.WHITE_DELTA):
            g.setColor(Color.white);
            drawArrow(g, deltaHead, point, otherEnd);
            break;
        case (EdgeView.BLACK_DELTA):
            g.setColor(Color.black);
            drawArrow(g, deltaHead, point, otherEnd);
            break;
        case (EdgeView.EDGE_COLOR_DELTA):
            g.setColor((Color) edge.get(Cerebral.FILL_COLOR));
            drawArrow(g, deltaHead, point, otherEnd);
            break;

        case (EdgeView.WHITE_DIAMOND):
            g.setColor(Color.white);
            drawArrow(g, diamondHead, point, otherEnd);
            break;
        case (EdgeView.BLACK_DIAMOND):
            g.setColor(Color.black);
            drawArrow(g, diamondHead, point, otherEnd);
            break;
        case (EdgeView.EDGE_COLOR_DIAMOND):
            g.setColor((Color) edge.get(Cerebral.FILL_COLOR));
            drawArrow(g, diamondHead, point, otherEnd);
            break;

        case (EdgeView.WHITE_ARROW):
            g.setColor(Color.white);
            drawArrow(g, arrowHead, point, otherEnd);
            break;
        case (EdgeView.BLACK_ARROW):
            g.setColor(Color.black);
            drawArrow(g, arrowHead, point, otherEnd);
            break;
        case (EdgeView.EDGE_COLOR_ARROW):
            g.setColor((Color) edge.get(Cerebral.FILL_COLOR));
            drawArrow(g, arrowHead, point, otherEnd);
            break;

        case (EdgeView.WHITE_T):
            g.setColor(Color.white);
            drawArrow(g, tHead, point, otherEnd);
            break;
        case (EdgeView.BLACK_T):
            g.setColor(Color.black);
            drawArrow(g, tHead, point, otherEnd);
            break;
        case (EdgeView.EDGE_COLOR_T):
            g.setColor((Color) edge.get(Cerebral.FILL_COLOR));
            drawArrow(g, tHead, point, otherEnd);
            break;

        }

    }

    protected Polygon getDeltaHead(int w, int h) {
        Polygon deltaHead = new Polygon();

        deltaHead.addPoint(0, h);
        deltaHead.addPoint(-w / 2, 0);
        deltaHead.addPoint(w / 2, 0);
        deltaHead.addPoint(0, h);
        return deltaHead;
    }

    protected Polygon getArrowHead(int w, int h) {
        Polygon arrowHead = new Polygon();

        arrowHead.addPoint(0, h);
        arrowHead.addPoint(-w / 2, 0);
        arrowHead.addPoint(0, h / 3);
        arrowHead.addPoint(w / 2, 0);
        arrowHead.addPoint(0, h);
        return arrowHead;

    }

    protected Polygon getT(int w) {
        Polygon t = new Polygon();
        t.addPoint(-w / 2, 1);
        t.addPoint(-w / 2, 0);
        t.addPoint(w / 2, 0);
        t.addPoint(w / 2, 1);
        t.addPoint(-w / 2, 1);

        return t;
    }

    private void drawArrow(Graphics2D g, Shape shape, Point2D p2, Point2D p1) {

        arrowTrans.setToTranslation(p2.getX(), p2.getY());
        arrowTrans.rotate(-(Math.PI / 2)
                + Math.atan2(p2.getY() - p1.getY(), p2.getX() - p1.getX()));
        curArrow = arrowTrans.createTransformedShape(shape);

        g.fill(curArrow);
    }

    private Polygon getDiamond(int w, int h) {
        Polygon diamond = new Polygon();
        diamond.addPoint(0, h);
        diamond.addPoint(-w / 2, h / 2);
        diamond.addPoint(0, 0);
        diamond.addPoint(w / 2, h / 2);
        diamond.addPoint(0, h);
        return diamond;
    }

}
