package cerebral.render;

import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import cerebral.Cerebral;

import prefuse.Display;
import prefuse.visual.VisualItem;

//Just like ShapeRenderer except we'll trigger mouse over events to an area twice as
//big as our object

//Also draws the starts of lines that join to this object over the object border in
// the color of this node.
public class EasyHoverShapeRenderer extends ExtendedShapeRenderer {

    protected Line2D m_line = new Line2D.Float();

    private Display d;

    private double scaleRange;

    private double minScale;

    public EasyHoverShapeRenderer(Display d) {
        this.d = d;

        Rectangle2D bounds = d.getVisualization().getBounds(Cerebral.NODES);
        double w = d.getWidth();
        double h = d.getHeight();
        double cx = bounds.getCenterX();
        double cy = bounds.getCenterY();

        // compute half-widths of final bounding box around
        // the desired center point
        double wb = Math.max(cx - bounds.getMinX(), bounds.getMaxX() - cx);
        double hb = Math.max(cy - bounds.getMinY(), bounds.getMaxY() - cy);

        // compute scale factor
        // - figure out if z or y dimension takes priority
        // - then balance against the current scale factor
        minScale = Math.min(w / (2 * wb), h / (2 * hb));
        scaleRange = 1.5 - minScale;
    }

    public void setBounds(VisualItem item) {
        super.setBounds(item);
        Rectangle2D bounds = (Rectangle2D) item.get(VisualItem.BOUNDS);
        double extraFactor = (1 - ((d.getScale() - minScale) / scaleRange)) / 2;
        if (extraFactor < 0)
            extraFactor = 0;
        if (extraFactor > 0.5)
            extraFactor = 0.5;

        double extraWidth = bounds.getWidth() * extraFactor;
        double extraHeight = bounds.getHeight() * extraFactor;
        bounds.setRect(bounds.getX() - extraWidth, bounds.getY() - extraHeight,
                bounds.getWidth() + 2 * (extraWidth), bounds.getHeight() + 2
                        * (extraHeight));

    }

    public boolean locatePoint(Point2D p, VisualItem item) {
        if (item.getBounds().contains(p)) {
            return true;
        }
        return false;
    }

    public void render(Graphics2D g, VisualItem item) {
        super.render(g, item);
    }

}
